@@ -1,11 +1,11 @@
 #define NAME          "IPhreeqc"
 #define VER_MAJOR      3
-#define VER_MINOR      6
-#define VER_PATCH      3
-#define VER_REVISION   15000
-#define GIT_COMMIT     572833907031c72cb6cabfca61fb0560251ea0d4
+#define VER_MINOR      7
+#define VER_PATCH      4
+#define VER_REVISION   16082
+#define GIT_COMMIT     f5cb62067c371d11e1cf9a04170ea342fea8300d
 
-#define RELEASE_DATE           "May 11, 2020"
+#define RELEASE_DATE           "March 26, 2022"
 
 #define APR_STRINGIFY(n) APR_STRINGIFY_HELPER(n)
 #define APR_STRINGIFY_HELPER(n) #n
