@@ -24,6 +24,14 @@
 #include "Surface.h"
 #include "phqalloc.h"
 
+#if defined(PHREEQCI_GUI)
+#ifdef _DEBUG
+#define new DEBUG_NEW
+#undef THIS_FILE
+static char THIS_FILE[] = __FILE__;
+#endif
+#endif
+
 /* ---------------------------------------------------------------------- */
 int Phreeqc::
 read_dump(void)
@@ -459,7 +467,7 @@ int Phreeqc::
 run_as_cells(void)
 /* ---------------------------------------------------------------------- */
 {
-	struct save save_data;
+	class save save_data;
 	LDBLE kin_time;
 	int count_steps, use_mix;
 	char token[2 * MAX_LENGTH];
@@ -534,7 +542,7 @@ run_as_cells(void)
 			/*
 			*  save data for saving solutions
 			*/
-			memcpy(&save_data, &save, sizeof(struct save));
+			memcpy(&save_data, &save, sizeof(class save));
 			/* 
 			*Copy everything to -2
 			*/
@@ -611,7 +619,7 @@ run_as_cells(void)
 			/*
 			*   save end of reaction
 			*/
-			memcpy(&save, &save_data, sizeof(struct save));
+			memcpy(&save, &save_data, sizeof(class save));
 			if (use.Get_kinetics_in() == TRUE)
 			{
 				Utilities::Rxn_copy(Rxn_kinetics_map, -2, use.Get_n_kinetics_user());
@@ -669,7 +677,7 @@ int Phreeqc::
 run_as_cells(void)
 /* ---------------------------------------------------------------------- */
 {
-	struct save save_data;
+	class save save_data;
 	LDBLE kin_time;
 	int count_steps, use_mix;
 	char token[2 * MAX_LENGTH];
@@ -745,7 +753,8 @@ run_as_cells(void)
 		/*
 		*  save data for saving solutions
 		*/
-		memcpy(&save_data, &save, sizeof(struct save));
+		// memcpy(&save_data, &save, sizeof(class save));
+		save_data = save;
 		/* 
 		*Copy everything to -2
 		*/
@@ -825,7 +834,8 @@ run_as_cells(void)
 		/*
 		*   save end of reaction
 		*/
-		memcpy(&save, &save_data, sizeof(struct save));
+		// memcpy(&save, &save_data, sizeof(class save));
+		save = save_data;
 		if (use.Get_kinetics_in() == TRUE)
 		{
 			Utilities::Rxn_copy(Rxn_kinetics_map, -2, use.Get_n_kinetics_user());
