@@ -12,6 +12,13 @@
 #include "cxxMix.h"
 #include "phqalloc.h"
 
+#if defined(PHREEQCI_GUI)
+#ifdef _DEBUG
+#define new DEBUG_NEW
+#undef THIS_FILE
+static char THIS_FILE[] = __FILE__;
+#endif
+#endif
 
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
@@ -96,74 +103,6 @@ cxxSurface::Get_related_rate() const
 	}
 	return (false);
 }
-#ifdef SKIP
-void
-cxxSurface::dump_xml(std::ostream & s_oss, unsigned int indent) const
-{
-	unsigned int i;
-	s_oss.precision(DBL_DIG - 1);
-	std::string indent0(""), indent1(""), indent2("");
-	for (i = 0; i < indent; ++i)
-		indent0.append(Utilities::INDENT);
-	for (i = 0; i < indent + 1; ++i)
-		indent1.append(Utilities::INDENT);
-	for (i = 0; i < indent + 2; ++i)
-		indent2.append(Utilities::INDENT);
-
-	// Surface element and attributes
-	s_oss << indent0;
-	s_oss << "<surface " << "\n";
-
-	s_oss << indent1;
-	s_oss << "surface_type=\"" << this->type << "\"" << "\n";
-
-	s_oss << indent1;
-	s_oss << "dl_type=\"" << this->dl_type << "\"" << "\n";
-
-	s_oss << indent1;
-	s_oss << "sites_units=\"" << this->sites_units << "\"" << "\n";
-
-	s_oss << indent1;
-	s_oss << "only_counter_ions=\"" << this->
-		only_counter_ions << "\"" << "\n";
-
-	s_oss << indent1;
-	s_oss << "thickness=\"" << this->thickness << "\"" << "\n";
-
-	s_oss << indent1;
-	s_oss << "debye_lengths=\"" << this->debye_lengths << "\"" << "\n";
-
-	s_oss << indent1;
-	s_oss << "DDL_viscosity=\"" << this->DDL_viscosity << "\"" << "\n";
-
-	s_oss << indent1;
-	s_oss << "DDL_limit=\"" << this->DDL_limit << "\"" << "\n";
-
-	s_oss << indent1;
-	s_oss << "transport=\"" << this->transport << "\"" << "\n";
-
-	// surface component structures
-	s_oss << indent1;
-	s_oss << "<component " << "\n";
-	{
-		for (std::map < std::string, cxxSurfaceComp >::const_iterator it =
-			 this->surfaceComps.begin(); it != this->surfaceComps.end(); ++it)
-		{
-			(*it).second.dump_xml(s_oss, indent + 2);
-		}
-	}
-	// surface charge structures
-	s_oss << indent1;
-	s_oss << "<charge_component " << "\n";
-	for (std::map < std::string, cxxSurfaceCharge >::const_iterator it =
-		 surface_charges.begin(); it != surface_charges.end(); ++it)
-	{
-		(*it).second.dump_xml(s_oss, indent + 2);
-	}
-
-	return;
-}
-#endif
 void
 cxxSurface::dump_raw(std::ostream & s_oss, unsigned int indent, int *n_out) const
 {
