@@ -15,6 +15,14 @@
 #include "Reaction.h"
 #include "Temperature.h"
 
+#if defined(PHREEQCI_GUI)
+#ifdef _DEBUG
+#define new DEBUG_NEW
+#undef THIS_FILE
+static char THIS_FILE[] = __FILE__;
+#endif
+#endif
+
 cxxSystem::cxxSystem(PHRQ_io *io)
 :
 PHRQ_base(io)
