@@ -1,5 +1,14 @@
 #include "UserPunch.h"
 #include "Phreeqc.h"
+
+#if defined(PHREEQCI_GUI)
+#ifdef _DEBUG
+#define new DEBUG_NEW
+#undef THIS_FILE
+static char THIS_FILE[] = __FILE__;
+#endif
+#endif
+
 UserPunch::UserPunch(int n, PHRQ_io *io)
 :	cxxNumKeyword(io)
 {
@@ -15,7 +24,7 @@ UserPunch::~UserPunch(void)
 		if (this->PhreeqcPtr != NULL)
 		{
 			this->PhreeqcPtr->rate_free(this->rate);
-			this->PhreeqcPtr->free_check_null(this->rate);
+			delete this->rate;
 		}
 	}
 	this->PhreeqcPtr = NULL;
