@@ -17,6 +17,14 @@
 #include <windows.h>
 #endif
 
+#if defined(PHREEQCI_GUI)
+#ifdef _DEBUG
+#define new DEBUG_NEW
+#undef THIS_FILE
+static char THIS_FILE[] = __FILE__;
+#endif
+#endif
+
 /* ---------------------------------------------------------------------- */
 void Phreeqc::
 initialize(void)
@@ -25,239 +33,35 @@ initialize(void)
 /*
  *   Initialize global variables
  */
-	struct logk *logk_ptr;
-	char token[MAX_LENGTH];
-
-	moles_per_kilogram_string = string_duplicate("Mol/kgw");
-	pe_string = string_duplicate("pe");
-/*
- *   Initialize advection
- */
-	advection_punch = (int *) PHRQ_malloc(sizeof(int));
-	if (advection_punch == NULL)
-		malloc_error();
-	advection_punch[0] = TRUE;
-	advection_print = (int *) PHRQ_malloc(sizeof(int));
-	if (advection_print == NULL)
-		malloc_error();
-	advection_print[0] = TRUE;
+	moles_per_kilogram_string = "Mol/kgw";
 /*
  *   Allocate space
  */
-	cell_data_max_cells = count_cells + 2;
-	space((void **) ((void *) &cell_data), INIT, &cell_data_max_cells,
-		  sizeof(struct cell_data));
-	for (int i = 0; i < cell_data_max_cells; i++)
-	{
-		cell_data[i].length = 1.0;
-		cell_data[i].mid_cell_x = 1.0;
-		cell_data[i].disp = 1.0;
-		cell_data[i].temp = 25.0;
-		cell_data[i].por = 0.1;
-		cell_data[i].por_il = 0.01;
-		cell_data[i].potV = 0;
-		cell_data[i].punch = FALSE;
-		cell_data[i].print = FALSE;
-	}
+	cell_data.resize((size_t)count_cells + 2); // initialized by global_structures.h
 
-	space((void **) ((void *) &elements), INIT, &max_elements,
-		  sizeof(struct element *));
-
-	space((void **) ((void *) &elt_list), INIT, &max_elts,
-		  sizeof(struct elt_list));
-
-	inverse = (struct inverse *) PHRQ_malloc((size_t) sizeof(struct inverse));
-	if (inverse == NULL) malloc_error();
 	count_inverse = 0;
 	space((void **) ((void *) &line), INIT, &max_line, sizeof(char));
 
 	space((void **) ((void *) &line_save), INIT, &max_line, sizeof(char));
 
-	space((void **) ((void *) &master), INIT, &max_master,
-		  sizeof(struct master *));
-
-	space((void **) ((void *) &mb_unknowns), INIT, &max_mb_unknowns,
-		  sizeof(struct unknown_list));
-
-	// one stag_data
-	stag_data = (struct stag_data *) PHRQ_calloc(1, sizeof(struct stag_data));
-	if (stag_data == NULL)
-		malloc_error();
-	stag_data->count_stag = 0;
-	stag_data->exch_f = 0;
-	stag_data->th_m = 0;
-	stag_data->th_im = 0;
-
-	space((void **) ((void *) &phases), INIT, &max_phases,
-		  sizeof(struct phase *));
-
-	space((void **) ((void *) &trxn.token), INIT, &max_trxn,
-		  sizeof(struct rxn_token_temp));
-
-	space((void **) ((void *) &s), INIT, &max_s, sizeof(struct species *));
-
-	space((void **) ((void *) &logk), INIT, &max_logk, sizeof(struct logk *));
-
-	space((void **) ((void *) &master_isotope), INIT, &max_master_isotope,
-		  sizeof(struct master_isotope *));
-
-/*
- *   Create hash tables
- */
-	hcreate_multi((unsigned) max_logk, &logk_hash_table);
-	hcreate_multi((unsigned) max_master_isotope, &master_isotope_hash_table);
-	hcreate_multi((unsigned) max_elements, &elements_hash_table);
-	hcreate_multi((unsigned) max_s, &species_hash_table);
-	hcreate_multi((unsigned) max_phases, &phases_hash_table);
-#ifdef SKIP_OLD_SELECTED_OUTPUT
-/*
- *   Initialize punch
- */
-	punch.in = FALSE;
-	punch.new_def = FALSE;
-
-	// one punch.totals
-	punch.count_totals = 0;
-	punch.totals =
-		(struct name_master *) PHRQ_malloc(sizeof(struct name_master));
-	if (punch.totals == NULL)
-		malloc_error();
-
-	// one punch.molalities
-	punch.count_molalities = 0;
-	punch.molalities =
-		(struct name_species *) PHRQ_malloc(sizeof(struct name_species));
-	if (punch.molalities == NULL)
-		malloc_error();
-
-	// one punch.activities
-	punch.count_activities = 0;
-	punch.activities =
-		(struct name_species *) PHRQ_malloc(sizeof(struct name_species));
-	if (punch.activities == NULL)
-		malloc_error();
-
-	// one punch.pure_phases
-	punch.count_pure_phases = 0;
-	punch.pure_phases =
-		(struct name_phase *) PHRQ_malloc(sizeof(struct name_phase));
-	if (punch.pure_phases == NULL)
-		malloc_error();
-
-	// one punch.si
-	punch.count_si = 0;
-	punch.si = (struct name_phase *) PHRQ_malloc(sizeof(struct name_phase));
-	if (punch.si == NULL)
-		malloc_error();
-
-	// one punch.gases
-	punch.count_gases = 0;
-	punch.gases =
-		(struct name_phase *) PHRQ_malloc(sizeof(struct name_phase));
-	if (punch.gases == NULL)
-		malloc_error();
-
-	// one punch.s_s
-	punch.count_s_s = 0;
-	punch.s_s = (struct name_phase *) PHRQ_malloc(sizeof(struct name_phase));
-	if (punch.s_s == NULL)
-		malloc_error();
-
-	// one punch.kinetics
-	punch.count_kinetics = 0;
-	punch.kinetics =
-		(struct name_phase *) PHRQ_malloc(sizeof(struct name_phase));
-	if (punch.kinetics == NULL)
-		malloc_error();
-
-	// one punch.isotopes
-	punch.count_isotopes = 0;
-	punch.isotopes =
-		(struct name_master *) PHRQ_malloc(sizeof(struct name_master));
-	if (punch.isotopes == NULL)
-		malloc_error();
-
-	// one punch.calculate_values
-	punch.count_calculate_values = 0;
-	punch.calculate_values =
-		(struct name_master *) PHRQ_malloc(sizeof(struct name_master));
-	if (punch.calculate_values == NULL)
-		malloc_error();
-#endif
-	// one save_values
-	save_values =
-		(struct save_values *) PHRQ_malloc(sizeof(struct save_values));
-	if (save_values == NULL)
-		malloc_error();
-
-	// one rate
-	rates = (struct rate *) PHRQ_malloc(sizeof(struct rate));
-	if (rates == NULL)
-		malloc_error();
+	// one stag_data in phreeqc.h, initialized in global_structures
 
 	// user_print
-	user_print = (struct rate *) PHRQ_malloc((size_t) sizeof(struct rate));
-	if (user_print == NULL)
-		malloc_error();
-	user_print->commands = NULL;
+	user_print = new class rate;
+	user_print->name = string_hsave("User_print");
+	user_print->commands.clear();
 	user_print->linebase = NULL;
 	user_print->varbase = NULL;
 	user_print->loopbase = NULL;
-
-#ifdef SKIP
-	// user_punch
-	user_punch = (struct rate *) PHRQ_malloc((size_t) sizeof(struct rate));
-	if (user_punch == NULL)
-		malloc_error();
-	user_punch->commands = NULL;
-	user_punch->linebase = NULL;
-	user_punch->varbase = NULL;
-	user_punch->loopbase = NULL;
-	user_punch_headings = (const char **) PHRQ_malloc(sizeof(char *));
-	if (user_punch_headings == NULL)
-		malloc_error();
-	user_punch_count_headings = 0;
-#endif
-#if defined PHREEQ98
-/*
- *   user_graph
- */
-	user_graph = (struct rate *) PHRQ_malloc((size_t) sizeof(struct rate));
-	if (user_graph == NULL)
-		malloc_error();
-	user_graph->commands = NULL;
-	user_graph->linebase = NULL;
-	user_graph->varbase = NULL;
-	user_graph->loopbase = NULL;
-	user_graph_headings = (char **) PHRQ_malloc(sizeof(char *));
-	if (user_graph_headings == NULL)
-		malloc_error();
-	user_graph_count_headings = 0;
-#endif
 	/*
 	   Initialize llnl aqueous model parameters
 	 */
-	llnl_temp = (LDBLE *) PHRQ_malloc(sizeof(LDBLE));
-	if (llnl_temp == NULL)
-		malloc_error();
-	llnl_count_temp = 0;
-	llnl_adh = (LDBLE *) PHRQ_malloc(sizeof(LDBLE));
-	if (llnl_adh == NULL)
-		malloc_error();
-	llnl_count_adh = 0;
-	llnl_bdh = (LDBLE *) PHRQ_malloc(sizeof(LDBLE));
-	if (llnl_bdh == NULL)
-		malloc_error();
-	llnl_count_bdh = 0;
-	llnl_bdot = (LDBLE *) PHRQ_malloc(sizeof(LDBLE));
-	if (llnl_bdot == NULL)
-		malloc_error();
-	llnl_count_bdot = 0;
-	llnl_co2_coefs = (LDBLE *) PHRQ_malloc(sizeof(LDBLE));
-	if (llnl_co2_coefs == NULL)
-		malloc_error();
-	llnl_count_co2_coefs = 0;
+	a_llnl = b_llnl = 0.0;
     // new PBasic
+	if (basic_interpreter != NULL)
+	{
+		basic_free();
+	}
 	basic_interpreter = new PBasic(this, phrq_io);
 	// allocate one change_surf
 	change_surf =
@@ -269,61 +73,21 @@ initialize(void)
 	change_surf[0].next = TRUE;
 	change_surf[1].cell_no = -99;
 	change_surf[1].next = FALSE;
+	/*
+	* define constant named log_k
+	*/
+	class logk* logk_ptr = logk_store("XconstantX", TRUE);
+	read_log_k_only("1.0", &logk_ptr->log_k[0]);
 
 #ifdef PHREEQCI_GUI
 	g_spread_sheet.heading            = NULL;
 	g_spread_sheet.units              = NULL;
-	g_spread_sheet.count_rows         = 0;
-	g_spread_sheet.rows               = NULL;
 	g_spread_sheet.defaults.units     = NULL;
-	g_spread_sheet.defaults.count_iso = 0;
-	g_spread_sheet.defaults.iso       = NULL;
 	g_spread_sheet.defaults.redox     = NULL;
+	assert(g_spread_sheet.rows.empty());
+	assert(g_spread_sheet.defaults.iso.empty());
 #endif
 
-	/* calculate_value */
-	max_calculate_value = MAX_ELTS;
-	count_calculate_value = 0;
-	space((void **) ((void *) &calculate_value), INIT, &max_calculate_value,
-		  sizeof(struct calculate_value *));
-	hcreate_multi((unsigned) max_calculate_value,
-				  &calculate_value_hash_table);
-
-	/* isotope_ratio */
-	max_isotope_ratio = MAX_ELTS;
-	count_isotope_ratio = 0;
-	space((void **) ((void *) &isotope_ratio), INIT, &max_isotope_ratio,
-		  sizeof(struct isotope_ratio *));
-	hcreate_multi((unsigned) max_isotope_ratio, &isotope_ratio_hash_table);
-
-	/* isotope_value */
-	max_isotope_alpha = MAX_ELTS;
-	count_isotope_alpha = 0;
-	space((void **) ((void *) &isotope_alpha), INIT, &max_isotope_alpha,
-		  sizeof(struct isotope_alpha *));
-	hcreate_multi((unsigned) max_isotope_alpha, &isotope_alpha_hash_table);
-
-	/*
-	 * define constant named log_k
-	 */
-	strcpy(token, "XconstantX");
-	logk_ptr = logk_store(token, TRUE);
-	strcpy(token, "1.0");
-	read_log_k_only(token, &logk_ptr->log_k[0]);
-
-	// allocate space for copier
-	copier_init(&copy_solution);
-	copier_init(&copy_pp_assemblage);
-	copier_init(&copy_exchange);
-	copier_init(&copy_surface);
-	copier_init(&copy_ss_assemblage);
-	copier_init(&copy_gas_phase);
-	copier_init(&copy_kinetics);
-	copier_init(&copy_mix);
-	copier_init(&copy_reaction);
-	copier_init(&copy_temperature);
-	copier_init(&copy_pressure);
-
 	// Initialize cvode
 	cvode_init();
 
@@ -333,12 +97,6 @@ initialize(void)
 	// Allocate space for sit
 	sit_init();
 
-	// Allocate zeros
-	zeros = (LDBLE *) PHRQ_malloc(sizeof(LDBLE));
-	if (zeros == NULL)
-		malloc_error();
-	zeros[0] = 0.0;
-	zeros_max = 1;
 	use_kinetics_limiter = false;
 
 	return;
@@ -634,6 +392,7 @@ initial_solutions(int print)
 			int count_iterations = 0;
 			std::string input_units = solution_ref.Get_initial_data()->Get_units();
 			cxxISolution *initial_data_ptr = solution_ref.Get_initial_data();
+			density_iterations = 0;
 			for (;;)
 			{
 				prep();
@@ -650,6 +409,7 @@ initial_solutions(int print)
 					set(TRUE);
 					converge = model();
 				}
+				density_iterations++;
 				if (solution_ref.Get_initial_data()->Get_calc_density())
 				{
 					solution_ref.Set_density(calc_dens());
@@ -678,9 +438,10 @@ initial_solutions(int print)
 			add_isotopes(solution_ref);
 			punch_all();
 			print_all();
+			density_iterations = 0;
 			/* free_model_allocs(); */
 // remove pr_in
-			for (int i = 0; i < count_unknowns; i++)
+			for (size_t i = 0; i < count_unknowns; i++)
 			{
 				if (x[i]->type == SOLUTION_PHASE_BOUNDARY)
 					x[i]->phase->pr_in = false;
@@ -709,26 +470,6 @@ initial_solutions(int print)
 	initial_solution_isotopes = FALSE;
 	return (OK);
 }
-#ifdef SKIP
-/* ---------------------------------------------------------------------- */
-int Phreeqc::
-solution_mix()
-/* ---------------------------------------------------------------------- */
-{
-/*
- *   Go through list of solution_mix, mix, save solutions
- */
-	std::map<int, cxxMix>::iterator it;
-	for (it = Rxn_solution_mix_map.begin(); it != Rxn_solution_mix_map.end(); it++)
-	{
-		cxxSolution sol(Rxn_solution_map, it->second, it->second.Get_n_user(), this->phrq_io);
-		Rxn_solution_map[it->second.Get_n_user()] = sol;
-		Utilities::Rxn_copies(Rxn_solution_map, it->second.Get_n_user(), it->second.Get_n_user_end());
-	}
-	Rxn_solution_mix_map.clear();
-	return OK;
-}
-#endif
 /* ---------------------------------------------------------------------- */
 int Phreeqc::
 initial_exchangers(int print)
@@ -829,8 +570,8 @@ initial_gas_phases(int print)
 	int converge, converge1;
 	int last, n_user, print1;
 	char token[2 * MAX_LENGTH];
-	struct phase *phase_ptr;
-	struct rxn_token *rxn_ptr;
+	class phase *phase_ptr;
+	class rxn_token *rxn_ptr;
 	LDBLE lp;
 	bool PR = false;
 
@@ -898,7 +639,7 @@ initial_gas_phases(int print)
 				if (phase_ptr->in == TRUE)
 				{
 					lp = -phase_ptr->lk;
-					for (rxn_ptr = phase_ptr->rxn_x->token + 1;
+					for (rxn_ptr = &phase_ptr->rxn_x.token[0] + 1;
 						 rxn_ptr->s != NULL; rxn_ptr++)
 					{
 						lp += rxn_ptr->s->la * rxn_ptr->coef;
@@ -1032,7 +773,7 @@ reactions(void)
  */
 	int count_steps, use_mix;
 	char token[2 * MAX_LENGTH];
-	struct save save_data;
+	class save save_data;
 	LDBLE kin_time;
 	cxxKinetics *kinetics_ptr;
 
@@ -1076,7 +817,8 @@ reactions(void)
 /*
  *  save data for saving solutions
  */
-	memcpy(&save_data, &save, sizeof(struct save));
+	// memcpy(&save_data, &save, sizeof(class save));
+	save_data = save;
 	/*
 	 *Copy everything to -2
 	 */
@@ -1139,7 +881,8 @@ reactions(void)
 /*
  *   save end of reaction
  */
-	memcpy(&save, &save_data, sizeof(struct save));
+	// memcpy(&save, &save_data, sizeof(class save));
+	save = save_data;
 	if (use.Get_kinetics_in() == TRUE)
 	{
 		Utilities::Rxn_copy(Rxn_kinetics_map, -2, use.Get_n_kinetics_user());
@@ -1163,7 +906,7 @@ reactions(void)
 //		{	
 //			cxxGasComp *gc_ptr = &(gas_phase_ptr->Get_gas_comps()[i]);
 //			int k;
-//			struct phase *phase_ptr = phase_bsearch(gc_ptr->Get_phase_name().c_str(), &k, FALSE);
+//			class phase *phase_ptr = phase_bsearch(gc_ptr->Get_phase_name().c_str(), &k, FALSE);
 //			assert(phase_ptr);
 //			phase_ptr->pr_in = false;
 //		}
@@ -1192,8 +935,7 @@ saver(void)
 	if (save.solution == TRUE)
 	{
 		sprintf(token, "Solution after simulation %d.", simulation);
-		description_x = (char *) free_check_null(description_x);
-		description_x = string_duplicate(token);
+		description_x = token;
 		n = save.n_solution_user;
 		xsolution_save(n);
 		for (i = save.n_solution_user + 1; i <= save.n_solution_user_end; i++)
@@ -1269,7 +1011,7 @@ xexchange_save(int n_user)
  *   Save exchanger assemblage into structure exchange with user
  *   number n_user.
  */
-	int i, j;
+	size_t i, j;
 	char token[MAX_LENGTH];
 
 	LDBLE charge;
@@ -1309,7 +1051,7 @@ xexchange_save(int n_user)
 			count_elts = 0;
 			paren_count = 0;
 			charge = 0.0;
-			for (j = 0; j < count_species_list; j++)
+			for (j = 0; j < species_list.size(); j++)
 			{
 				if (species_list[j].master_s == x[i]->master[0]->s)
 				{
@@ -1351,19 +1093,19 @@ int Phreeqc::
 xgas_save(int n_user)
 /* ---------------------------------------------------------------------- */
 {
-/*
- *   Save gas composition into structure gas_phase with user
- *   number n_user.
- */
+	/*
+	 *   Save gas composition into structure gas_phase with user
+	 *   number n_user.
+	 */
 	char token[MAX_LENGTH];
 
 	if (use.Get_gas_phase_ptr() == NULL)
 		return (OK);
-	cxxGasPhase *gas_phase_ptr = use.Get_gas_phase_ptr();
+	cxxGasPhase* gas_phase_ptr = use.Get_gas_phase_ptr();
 	cxxGasPhase temp_gas_phase(*gas_phase_ptr);
-/*
- *   Store in gas_phase
- */
+	/*
+	 *   Store in gas_phase
+	 */
 	temp_gas_phase.Set_n_user(n_user);
 	temp_gas_phase.Set_n_user_end(n_user);
 	sprintf(token, "Gas phase after simulation %d.", simulation);
@@ -1371,22 +1113,38 @@ xgas_save(int n_user)
 	temp_gas_phase.Set_new_def(false);
 	temp_gas_phase.Set_solution_equilibria(false);
 	temp_gas_phase.Set_n_solution(-99);
-/*
- *   Update amounts
- */
-	for (size_t i = 0 ; i < temp_gas_phase.Get_gas_comps().size(); i++)
+	/*
+	 *   Update amounts
+	 */
+	bool PR = false;
+	if (gas_phase_ptr->Get_v_m() >= 0.01) PR = true;
+	for (size_t i = 0; i < temp_gas_phase.Get_gas_comps().size(); i++)
 	{
-		cxxGasComp * gc_ptr = &(temp_gas_phase.Get_gas_comps()[i]);
+		cxxGasComp* gc_ptr = &(temp_gas_phase.Get_gas_comps()[i]);
 		int k;
-		struct phase *phase_ptr = phase_bsearch(gc_ptr->Get_phase_name().c_str(), &k, FALSE);
+		class phase* phase_ptr = phase_bsearch(gc_ptr->Get_phase_name().c_str(), &k, FALSE);
 		assert(phase_ptr);
-		gc_ptr->Set_moles(phase_ptr->moles_x);
+		if (PR)
+		{
+			gc_ptr->Set_moles(phase_ptr->moles_x);
+			gc_ptr->Set_p(phase_ptr->p_soln_x);
+			gc_ptr->Set_phi(phase_ptr->pr_phi);
+			gc_ptr->Set_f(phase_ptr->p_soln_x * phase_ptr->pr_phi);
+		}
+		else
+		{
+			gc_ptr->Set_moles(phase_ptr->moles_x);
+			gc_ptr->Set_p(phase_ptr->p_soln_x);
+			gc_ptr->Set_phi(1.0);
+			gc_ptr->Set_f(phase_ptr->p_soln_x);
+		}
 	}
 	Rxn_gas_phase_map[n_user] = temp_gas_phase;
 
 	use.Set_gas_phase_ptr(NULL);
 	return (OK);
 }
+
 /* ---------------------------------------------------------------------- */
 int Phreeqc::
 xss_assemblage_save(int n_user)
@@ -1482,7 +1240,7 @@ xsolution_save(int n_user)
  *
  *   input:  n_user is user solution number of target
  */
-	struct master *master_i_ptr, *master_ptr;
+	class master *master_i_ptr, *master_ptr;
 /*
  *   Malloc space for solution data
  */
@@ -1514,7 +1272,7 @@ xsolution_save(int n_user)
 	 */
 	if (initial_solution_isotopes == TRUE)
 	{
-		for (int i = 0; i < count_master_isotope; i++)
+		for (int i = 0; i < (int)master_isotope.size(); i++)
 		{
 			if (master_isotope[i]->moles > 0)
 			{
@@ -1555,7 +1313,7 @@ xsolution_save(int n_user)
 /*
  *   Copy totals data
  */
-	for (int i = 0; i < count_master; i++)
+	for (int i = 0; i < (int)master.size(); i++)
 	{
 		if (master[i]->s->type == EX ||
 			master[i]->s->type == SURF || master[i]->s->type == SURF_PSI)
@@ -1584,7 +1342,7 @@ xsolution_save(int n_user)
 	}
 	if (pitzer_model == TRUE || sit_model == TRUE)
 	{
-		for (int j = 0; j < count_s_x; j++)
+		for (int j = 0; j < (int)this->s_x.size(); j++)
 		{
 			if (s_x[j]->lg != 0.0)
 			{
@@ -1599,7 +1357,7 @@ xsolution_save(int n_user)
 	std::map< std::string, cxxSolutionIsotope >::iterator it;
 	for (it = temp_solution.Get_isotopes().begin(); it != temp_solution.Get_isotopes().end(); it++)
 	{
-		struct master *iso_master_ptr = master_bsearch(it->second.Get_elt_name().c_str());
+		class master *iso_master_ptr = master_bsearch(it->second.Get_elt_name().c_str());
 		if (iso_master_ptr != NULL)
 		{
 			it->second.Set_total(iso_master_ptr->total);
@@ -1619,42 +1377,11 @@ xsolution_save(int n_user)
 			warning_msg(error_string);
 		}
 	}
-#ifdef SKIP
-/*
- * Bug-fix
- * Create and initialize intial data (this object should always be present even if it is left empty)
- */
- 
-   temp_solution.Create_initial_data();
-   cxxISolution* initialData = temp_solution.Get_initial_data();
-
-   initialData->Set_units( "Mol/kgw" );
-
-   // Copy totals to initialdata when present
-   if ( !temp_solution.Get_totals().empty() )
-   {
-      cxxNameDouble& totals = temp_solution.Get_totals();
-
-      for (cxxNameDouble::iterator jit = totals.begin(); jit != totals.end(); jit++)
-      {
-         std::string compName( jit->first );
-         double compConc = jit->second;
-
-         SolutionCompMap& comps = initialData->Get_comps();
-
-         cxxISolutionComp& tempComp = comps[ compName ];
-
-         tempComp.Set_description( compName.c_str() );
-         tempComp.Set_input_conc( compConc / temp_solution.Get_mass_water());
-         tempComp.Set_units( initialData->Get_units().c_str() );
-      }
-   } 
-#endif 
    if (this->save_species)
    {
 	   // saves mol/L
 	   temp_solution.Get_species_map().clear();
-	   for (int i = 0; i < this->count_s_x; i++)
+	   for (int i = 0; i < (int)this->s_x.size(); i++)
 	   {
 		   if (s_x[i]->type <= H2O)
 		   {
@@ -1663,13 +1390,22 @@ xsolution_save(int n_user)
 	   }	 
 	   // saves gamma
 	   temp_solution.Get_log_gamma_map().clear();
-	   for (int i = 0; i < this->count_s_x; i++)
+	   for (int i = 0; i < (int)this->s_x.size(); i++)
 	   {
 		   if (s_x[i]->type <= H2O)
 		   {
 			   temp_solution.Get_log_gamma_map()[s_x[i]->number] = s_x[i]->lg;
 		   }
 	   }
+	   // saves molalities
+	   temp_solution.Get_log_molalities_map().clear();
+	   for (int i = 0; i < (int)this->s_x.size(); i++)
+	   {
+		   if (s_x[i]->type <= H2O)
+		   {
+			   temp_solution.Get_log_molalities_map()[s_x[i]->number] = s_x[i]->lm;
+		   }
+	   }
    }
 /*
  *   Save solution
@@ -1730,7 +1466,7 @@ xsurface_save(int n_user)
 			count_elts = 0;
 			paren_count = 0;
 			charge = 0.0;
-			for (int j = 0; j < count_species_list; j++)
+			for (int j = 0; j < (int)species_list.size(); j++)
 			{
 				if (species_list[j].master_s == x[i]->master[0]->s)
 				{
@@ -1808,7 +1544,7 @@ xsurface_save(int n_user)
 		{
 			cxxSurfaceCharge & charge_ref = surface_ptr->Get_surface_charges()[i];
 			double mass_water_surface = charge_ref.Get_mass_water();
-			for (int j = 0; j < count_s_x; j++)
+			for (int j = 0; j < (int)this->s_x.size(); j++)
 			{
 				if (s_x[j]->type > H2O)
 					continue;
@@ -1816,7 +1552,6 @@ xsurface_save(int n_user)
 				double moles_excess = mass_water_aq_x * molality * charge_ref.Get_g_map()[s_x[j]->z].Get_g();
 				double moles_surface = mass_water_surface * molality + moles_excess;
 				charge_ref.Get_dl_species_map()[s_x[j]->number] = moles_surface/mass_water_surface;
-//#ifdef SKIP
 				double g = charge_ref.Get_g_map()[s_x[j]->z].Get_g();
 				//double moles_excess = mass_water_aq_x * molality * (g * s_x[j]->erm_ddl +
 				//	mass_water_surface /
@@ -1832,9 +1567,6 @@ xsurface_save(int n_user)
 				moles_excess = mass_water_aq_x * molality * g;
 				double c = (mass_water_surface * molality + moles_excess) / mass_water_surface;
 				charge_ref.Get_dl_species_map()[s_x[j]->number] = c;
-
-
-//#endif
 			}
 			//charge_ref.Get_dl_species_map()[s_h2o->number] = 0.0;
 			charge_ref.Get_dl_species_map()[s_h2o->number] = 1.0/gfw_water;
@@ -2015,7 +1747,7 @@ step_save_exch(int n_user)
 	}
 
 	// Set exchange total in one component
-	for (int i = 0; i < count_master; i++)
+	for (int i = 0; i < (int)master.size(); i++)
 	{
 		if (master[i]->s->type != EX)
 			continue;
@@ -2060,7 +1792,7 @@ step_save_surf(int n_user)
 		return (OK);
 	Utilities::Rxn_copy(Rxn_surface_map, use.Get_surface_ptr()->Get_n_user(), n_user);
 	cxxSurface *surface_ptr = Utilities::Rxn_find(Rxn_surface_map, n_user);
-	for (int i = 0; i < count_master; i++)
+	for (int i = 0; i < (int)master.size(); i++)
 	{
 		if (master[i]->s->type != SURF)
 			continue;
@@ -2117,286 +1849,164 @@ int Phreeqc::
 copy_entities(void)
 /* ---------------------------------------------------------------------- */
 {
-	int i, j, return_value;
-	int verbose;
-
-	verbose = FALSE;
+	int return_value;
 	return_value = OK;
-	if (copy_solution.count > 0)
+	for (size_t j = 0; j < copy_solution.n_user.size(); j++)
 	{
-		for (j = 0; j < copy_solution.count; j++)
+		if (Utilities::Rxn_find(Rxn_solution_map, copy_solution.n_user[j]) != NULL)
 		{
-			if (Utilities::Rxn_find(Rxn_solution_map, copy_solution.n_user[j]) != NULL)
+			for (size_t i = copy_solution.start[j]; i <= copy_solution.end[j]; i++)
 			{
-				for (i = copy_solution.start[j]; i <= copy_solution.end[j];
-					i++)
-				{
-					if (i == copy_solution.n_user[j])
-						continue;
-					Utilities::Rxn_copy(Rxn_solution_map, copy_solution.n_user[j], i);
-				}
-			}
-			else
-			{
-				if (verbose == TRUE)
-				{
-					warning_msg("SOLUTION to copy not found.");
-					return_value = ERROR;
-				}
-			}
-		}
-	}
-	if (copy_pp_assemblage.count > 0)
-	{
-		for (j = 0; j < copy_pp_assemblage.count; j++)
-		{
-			if (Utilities::Rxn_find(Rxn_pp_assemblage_map, copy_pp_assemblage.n_user[j]) != NULL)
-			{
-				for (i = copy_pp_assemblage.start[j];
-					i <= copy_pp_assemblage.end[j]; i++)
-				{
-					if (i == copy_pp_assemblage.n_user[j])
-						continue;
-					Utilities::Rxn_copy(Rxn_pp_assemblage_map, copy_pp_assemblage.n_user[j], i);
-				}
-			}
-			else
-			{
-				if (verbose == TRUE)
-				{
-					warning_msg("EQUILIBRIUM_PHASES to copy not found.");
-					return_value = ERROR;
-				}
-			}
-		}
-	}
-	if (copy_reaction.count > 0)
-	{
-		for (j = 0; j < copy_reaction.count; j++)
-		{
-			if (Utilities::Rxn_find(Rxn_reaction_map, copy_reaction.n_user[j]) != NULL)
-			{
-				for (i = copy_reaction.start[j]; i <= copy_reaction.end[j]; i++)
-				{
-					if (i == copy_reaction.n_user[j])
-						continue;
-					Utilities::Rxn_copy(Rxn_reaction_map, copy_reaction.n_user[j], i);
-				}
-			}
-			else
-			{
-				if (verbose == TRUE)
-				{
-					warning_msg("REACTION to copy not found.");
-					return_value = ERROR;
-				}
-			}
-		}
-	}
-	if (copy_mix.count > 0)
-	{
-		for (j = 0; j < copy_mix.count; j++)
-		{
-			if (Utilities::Rxn_find(Rxn_mix_map, copy_mix.n_user[j]) != NULL)
-			{
-				for (i = copy_mix.start[j]; i <= copy_mix.end[j]; i++)
-				{
-					if (i != copy_mix.n_user[j])
-					{
-						Utilities::Rxn_copy(Rxn_mix_map, copy_mix.n_user[j], i);
-					}
-				}
-			}
-			else
-			{
-				if (verbose == TRUE)
-				{
-					warning_msg("Mix to copy not found.");
-					return_value = ERROR;
-				}
+				if (i == copy_solution.n_user[j])
+					continue;
+				Utilities::Rxn_copy(Rxn_solution_map, copy_solution.n_user[j], (int)i);
 			}
 		}
 	}
+	copier_clear(&copy_solution);
 
-	if (copy_exchange.count > 0)
+	for (size_t j = 0; j < copy_pp_assemblage.n_user.size(); j++)
 	{
-		for (j = 0; j < copy_exchange.count; j++)
+		if (Utilities::Rxn_find(Rxn_pp_assemblage_map, copy_pp_assemblage.n_user[j]) != NULL)
 		{
-			if (Utilities::Rxn_find(Rxn_exchange_map, copy_exchange.n_user[j]) != NULL)
+			for (size_t i = copy_pp_assemblage.start[j]; i <= copy_pp_assemblage.end[j]; i++)
 			{
-				for (i = copy_exchange.start[j]; i <= copy_exchange.end[j];
-					i++)
-				{
-					if (i == copy_exchange.n_user[j])
-						continue;
-					Utilities::Rxn_copy(Rxn_exchange_map, copy_exchange.n_user[j], i);
-				}
-			}
-			else
-			{
-				if (verbose == TRUE)
-				{
-					warning_msg("EXCHANGE to copy not found.");
-					return_value = ERROR;
-				}
-			}
-		}
-	}
-	if (copy_surface.count > 0)
-	{
-		for (j = 0; j < copy_surface.count; j++)
-		{
-			if (Utilities::Rxn_find(Rxn_surface_map, copy_surface.n_user[j]) != NULL)
-			{
-				for (i = copy_surface.start[j]; i <= copy_surface.end[j]; i++)
-				{
-					if (i == copy_surface.n_user[j])
-						continue;
-					Utilities::Rxn_copy(Rxn_surface_map, copy_surface.n_user[j], i);
-				}
-			}
-			else
-			{
-				if (verbose == TRUE)
-				{
-					warning_msg("SURFACE to copy not found.");
-					return_value = ERROR;
-				}
+				if (i == copy_pp_assemblage.n_user[j])
+					continue;
+				Utilities::Rxn_copy(Rxn_pp_assemblage_map, copy_pp_assemblage.n_user[j], (int)i);
 			}
 		}
 	}
+	copier_clear(&copy_pp_assemblage);
 
-	if (copy_temperature.count > 0)
+	for (size_t j = 0; j < copy_reaction.n_user.size(); j++)
 	{
-		for (j = 0; j < copy_temperature.count; j++)
+		if (Utilities::Rxn_find(Rxn_reaction_map, copy_reaction.n_user[j]) != NULL)
 		{
-			if (Utilities::Rxn_find(Rxn_temperature_map, copy_temperature.n_user[j]) != NULL)
+			for (size_t i = copy_reaction.start[j]; i <= copy_reaction.end[j]; i++)
 			{
-				for (i = copy_temperature.start[j]; i <= copy_temperature.end[j]; i++)
-				{
-					if (i != copy_temperature.n_user[j])
-					{
-						Utilities::Rxn_copy(Rxn_temperature_map, copy_temperature.n_user[j], i);
-					}
-				}
+				if (i == copy_reaction.n_user[j])
+					continue;
+				Utilities::Rxn_copy(Rxn_reaction_map, copy_reaction.n_user[j], (int)i);
 			}
-			else
+		}
+	}
+	copier_clear(&copy_reaction);
+
+	for (size_t j = 0; j < copy_mix.n_user.size(); j++)
+	{
+		if (Utilities::Rxn_find(Rxn_mix_map, copy_mix.n_user[j]) != NULL)
+		{
+			for (size_t i = copy_mix.start[j]; i <= copy_mix.end[j]; i++)
 			{
-				if (verbose == TRUE)
+				if (i != copy_mix.n_user[j])
 				{
-					warning_msg("temperature to copy not found.");
-					return_value = ERROR;
+					Utilities::Rxn_copy(Rxn_mix_map, copy_mix.n_user[j], (int)i);
 				}
 			}
 		}
 	}
-	if (copy_pressure.count > 0)
+	copier_clear(&copy_mix);
+
+	for (size_t j = 0; j < copy_exchange.n_user.size(); j++)
 	{
-		for (j = 0; j < copy_pressure.count; j++)
+		if (Utilities::Rxn_find(Rxn_exchange_map, copy_exchange.n_user[j]) != NULL)
 		{
-			if (Utilities::Rxn_find(Rxn_pressure_map, copy_pressure.n_user[j]) != NULL)
+			for (size_t i = copy_exchange.start[j]; i <= copy_exchange.end[j]; i++)
 			{
-				for (i = copy_pressure.start[j]; i <= copy_pressure.end[j]; i++)
-				{
-					if (i != copy_pressure.n_user[j])
-					{
-						Utilities::Rxn_copy(Rxn_pressure_map, copy_pressure.n_user[j], i);
-					}
-				}
+				if (i == copy_exchange.n_user[j]) continue;
+				Utilities::Rxn_copy(Rxn_exchange_map, copy_exchange.n_user[j], (int)i);
 			}
-			else
+		}
+	}
+	copier_clear(&copy_exchange);
+
+	for (size_t j = 0; j < copy_surface.n_user.size(); j++)
+	{
+		if (Utilities::Rxn_find(Rxn_surface_map, copy_surface.n_user[j]) != NULL)
+		{
+			for (size_t i = copy_surface.start[j]; i <= copy_surface.end[j]; i++)
 			{
-				if (verbose == TRUE)
+				if (i == copy_surface.n_user[j])
+					continue;
+				Utilities::Rxn_copy(Rxn_surface_map, copy_surface.n_user[j], (int)i);
+			}
+		}
+	}
+	copier_clear(&copy_surface);
+
+	for (size_t j = 0; j < copy_temperature.n_user.size(); j++)
+	{
+		if (Utilities::Rxn_find(Rxn_temperature_map, copy_temperature.n_user[j]) != NULL)
+		{
+			for (size_t i = copy_temperature.start[j]; i <= copy_temperature.end[j]; i++)
+			{
+				if (i != copy_temperature.n_user[j])
 				{
-					warning_msg("pressure to copy not found.");
-					return_value = ERROR;
+					Utilities::Rxn_copy(Rxn_temperature_map, copy_temperature.n_user[j], (int)i);
 				}
 			}
 		}
 	}
-	if (copy_gas_phase.count > 0)
+	copier_clear(&copy_temperature);
+
+	for (size_t j = 0; j < copy_pressure.n_user.size(); j++)
 	{
-		for (j = 0; j < copy_gas_phase.count; j++)
+		if (Utilities::Rxn_find(Rxn_pressure_map, copy_pressure.n_user[j]) != NULL)
 		{
-			if (Utilities::Rxn_find(Rxn_gas_phase_map, copy_gas_phase.n_user[j]) != NULL)
+			for (size_t i = copy_pressure.start[j]; i <= copy_pressure.end[j]; i++)
 			{
-				for (i = copy_gas_phase.start[j]; i <= copy_gas_phase.end[j];
-					i++)
+				if (i != copy_pressure.n_user[j])
 				{
-					if (i == copy_gas_phase.n_user[j])
-						continue;
-					Utilities::Rxn_copy(Rxn_gas_phase_map, copy_gas_phase.n_user[j], i);
-				}
-			}
-			else
-			{
-				if (verbose == TRUE)
-				{
-					warning_msg("EXCHANGE to copy not found.");
-					return_value = ERROR;
+					Utilities::Rxn_copy(Rxn_pressure_map, copy_pressure.n_user[j], (int)i);
 				}
 			}
 		}
 	}
-	if (copy_kinetics.count > 0)
+	copier_clear(&copy_pressure);
+
+	for (size_t j = 0; j < copy_gas_phase.n_user.size(); j++)
 	{
-		for (j = 0; j < copy_kinetics.count; j++)
+		if (Utilities::Rxn_find(Rxn_gas_phase_map, copy_gas_phase.n_user[j]) != NULL)
 		{
-			if (Utilities::Rxn_find(Rxn_kinetics_map, copy_kinetics.n_user[j]) != NULL)
+			for (size_t i = copy_gas_phase.start[j]; i <= copy_gas_phase.end[j]; i++)
 			{
-				for (i = copy_kinetics.start[j]; i <= copy_kinetics.end[j];
-					i++)
-				{
-					if (i == copy_kinetics.n_user[j])
-						continue;
-					Utilities::Rxn_copy(Rxn_kinetics_map, copy_kinetics.n_user[j], i);
-				}
-			}
-			else
-			{
-				if (verbose == TRUE)
-				{
-					warning_msg("KINETICS to copy not found.");
-					return_value = ERROR;
-				}
+				if (i == copy_gas_phase.n_user[j])
+					continue;
+				Utilities::Rxn_copy(Rxn_gas_phase_map, copy_gas_phase.n_user[j], (int)i);
 			}
 		}
 	}
-	if (copy_ss_assemblage.count > 0)
+	copier_clear(&copy_gas_phase);
+
+	for (size_t j = 0; j < copy_kinetics.n_user.size(); j++)
 	{
-		for (j = 0; j < copy_ss_assemblage.count; j++)
+		if (Utilities::Rxn_find(Rxn_kinetics_map, copy_kinetics.n_user[j]) != NULL)
 		{
-			if (Utilities::Rxn_find(Rxn_ss_assemblage_map, copy_ss_assemblage.n_user[j]) != NULL)
+			for (size_t i = copy_kinetics.start[j]; i <= copy_kinetics.end[j]; i++)
 			{
-				for (i = copy_ss_assemblage.start[j];
-					i <= copy_ss_assemblage.end[j]; i++)
-				{
-					if (i == copy_ss_assemblage.n_user[j])
-						continue;
-					Utilities::Rxn_copy(Rxn_ss_assemblage_map, copy_ss_assemblage.n_user[j], i);
-				}
-			}
-			else
-			{
-				if (verbose == TRUE)
-				{
-					warning_msg("SOLID_SOLUTIONS to copy not found.");
-					return_value = ERROR;
-				}
+				if (i == copy_kinetics.n_user[j])
+					continue;
+				Utilities::Rxn_copy(Rxn_kinetics_map, copy_kinetics.n_user[j], (int)i);
 			}
 		}
 	}
-	copy_solution.count = 0;
-	copy_pp_assemblage.count = 0;
-	copy_exchange.count = 0;
-	copy_surface.count = 0;
-	copy_ss_assemblage.count = 0;
-	copy_gas_phase.count = 0;
-	copy_kinetics.count = 0;
-	copy_mix.count = 0;
-	copy_reaction.count = 0;
-	copy_temperature.count = 0;
-	copy_pressure.count = 0;
+	copier_clear(&copy_kinetics);
+
+	for (size_t j = 0; j < copy_ss_assemblage.n_user.size(); j++)
+	{
+		if (Utilities::Rxn_find(Rxn_ss_assemblage_map, copy_ss_assemblage.n_user[j]) != NULL)
+		{
+			for (size_t i = copy_ss_assemblage.start[j]; i <= copy_ss_assemblage.end[j]; i++)
+			{
+				if (i == copy_ss_assemblage.n_user[j])
+					continue;
+				Utilities::Rxn_copy(Rxn_ss_assemblage_map, copy_ss_assemblage.n_user[j], (int)i);
+			}
+		}
+	}
+	copier_clear(&copy_ss_assemblage);
+
 	new_copy = FALSE;
 	return return_value;
 }
@@ -2433,12 +2043,10 @@ run_simulations(void)
 /* ---------------------------------------------------------------------- */
 {
 	char token[MAX_LENGTH];
-//#ifdef SKIP_KEEP
 #if defined(_MSC_VER) && (_MSC_VER < 1900)  // removed in vs2015
 	unsigned int old_exponent_format;
 	old_exponent_format = _set_output_format(_TWO_DIGIT_EXPONENT);
 #endif
-//#endif
 /*
  *   Prepare error handling
  */
@@ -2495,19 +2103,16 @@ run_simulations(void)
 			if (read_input() == EOF)
 				break;
 
-			if (title_x != NULL)
+			if (title_x.size() > 0)
 			{
 				sprintf(token, "TITLE");
 				dup_print(token, TRUE);
 				if (pr.headings == TRUE)
-					output_msg(sformatf( "%s\n\n", title_x));
+				{
+					output_msg(sformatf("%s\n\n", title_x.c_str()));
+				}
 			}
 			tidy_model();
-#ifdef PHREEQ98
-			if (!phreeq98_debug)
-			{
-#endif
-
 /*
  *   Calculate distribution of species for initial solutions
  */
@@ -2582,9 +2187,6 @@ run_simulations(void)
 			dup_print("End of simulation.", TRUE);
 			output_flush();
 			error_flush();
-#ifdef PHREEQ98
-		}						/* if (!phreeq98_debug) */
-#endif
 		}
 	}
 	catch (const PhreeqcStop&)
@@ -2631,9 +2233,11 @@ do_status(void)
 			screen_msg("\n");
 		}
 		//pr.headings = TRUE; // set in class_main; not set for IPhreeqc
-		LDBLE ext = (double) clock() / CLOCKS_PER_SEC;
+#ifndef TESTING
+		LDBLE ext = (double)clock() / CLOCKS_PER_SEC;
 		dup_print(sformatf("End of Run after %g Seconds.", ext), TRUE);
 		screen_msg(sformatf("\nEnd of Run after %g Seconds.\n", ext));
+#endif
 // appt this gives output when the charts are active...
 		phrq_io->output_flush();
 		phrq_io->error_flush();
@@ -2678,6 +2282,11 @@ save_init(int i)
 void
 Phreeqc::do_mixes(void)
 {
+	bool surf, exch, kin, min;
+	surf = (Rxn_surface_mix_map.size() > 0);
+	exch = (Rxn_exchange_mix_map.size() > 0);
+	kin = (Rxn_kinetics_mix_map.size() > 0);
+	min = (Rxn_pp_assemblage_mix_map.size() > 0);
 	Utilities::Rxn_mix(Rxn_solution_mix_map, Rxn_solution_map, this);
 	Utilities::Rxn_mix(Rxn_exchange_mix_map, Rxn_exchange_map, this);
 	Utilities::Rxn_mix(Rxn_gas_phase_mix_map, Rxn_gas_phase_map, this);
@@ -2685,4 +2294,8 @@ Phreeqc::do_mixes(void)
 	Utilities::Rxn_mix(Rxn_pp_assemblage_mix_map, Rxn_pp_assemblage_map, this);
 	Utilities::Rxn_mix(Rxn_ss_assemblage_mix_map, Rxn_ss_assemblage_map, this);
 	Utilities::Rxn_mix(Rxn_surface_mix_map, Rxn_surface_map, this);
+	if (exch || kin) update_kin_exchange();
+	if (exch || min) update_min_exchange();
+	if (surf || min) update_min_surface();
+	if (surf || kin) update_kin_surface();
 }
