@@ -1,12 +1,12 @@
 #include <iostream>				/* std::cout std::cerr */
 #include <sstream>
 #include <fstream>
+#include "Phreeqc.h"
 #include "StorageBin.h"
 #include "SS.h"
 #ifndef boolean
 typedef unsigned char boolean;
 #endif
-#include "Phreeqc.h"
 #include "phqalloc.h"
 #include "Utils.h"
 
@@ -17,6 +17,14 @@ typedef unsigned char boolean;
 #define OPTION_DEFAULT -4
 #define OPTION_DEFAULT2 -5
 
+#if defined(PHREEQCI_GUI)
+#ifdef _DEBUG
+#define new DEBUG_NEW
+#undef THIS_FILE
+static char THIS_FILE[] = __FILE__;
+#endif
+#endif
+
 /* ---------------------------------------------------------------------- */
 int Phreeqc::
 read_transport(void)
@@ -35,18 +43,16 @@ read_transport(void)
 	*         ERROR   if error occurred reading data
 	*
 	*/
-	char *ptr;
 	int i, j, l;
 	int count_length, count_disp, count_punch, count_print, count_por, count_same_model;
 	int count_length_alloc, count_disp_alloc, count_por_alloc;
 	char token[MAX_LENGTH];
-	char *description;
-	int n_user, n_user_end;
 	LDBLE *length, *disp, *pors;
 	int *punch_temp, *print_temp, *same_model_temp;
 	int return_value, opt, opt_save;
-	char *next_char, *next_char_save;
-	char file_name[MAX_LENGTH];
+	const char* next_char;
+	//char file_name[MAX_LENGTH];
+	std::string file_name("phreeqc.dmp");
 
 	const char *opt_list[] = {
 		"cells",				/* 0 */
@@ -100,7 +106,6 @@ read_transport(void)
 	};
 	int count_opt_list = 48;
 
-	strcpy(file_name, "phreeqc.dmp");
 	/*
 	*   Initialize
 	*/
@@ -143,12 +148,6 @@ read_transport(void)
 	count_length_alloc = count_disp_alloc = count_por_alloc = 1;
 	transport_start = 1;
 	/*
-	*   Read transport number (not currently used)
-	*/
-	ptr = line;
-	read_number_description(ptr, &n_user, &n_user_end, &description);
-	description = (char *)free_check_null(description);
-	/*
 	*   Set use data to last read
 	*/
 	use.Set_trans_in(true);
@@ -177,12 +176,12 @@ read_transport(void)
 			error_msg(line_save, CONTINUE);
 			break;
 		case 0:				/* cells */
-			sscanf(next_char, "%d", &count_cells);
+			(void)sscanf(next_char, "%d", &count_cells);
 			opt_save = OPTION_DEFAULT;
 			break;
 		case 1:				/* shifts */
 			if (copy_token(token, &next_char, &l) == DIGIT)
-				sscanf(token, "%d", &count_shifts);
+				(void)sscanf(token, "%d", &count_shifts);
 			else
 			{
 				warning_msg
@@ -193,7 +192,7 @@ read_transport(void)
 			if (j != EMPTY)
 			{
 				if (j == DIGIT)
-					sscanf(token, "%d", &ishift);
+					(void)sscanf(token, "%d", &ishift);
 				else
 				{
 					input_error++;
@@ -215,7 +214,7 @@ read_transport(void)
 		case 3:				/* selected_output */
 		case 29:				/* selected_output_frequency */
 		case 33:				/* punch_frequency */
-			sscanf(next_char, "%d", &punch_modulus);
+			(void)sscanf(next_char, "%d", &punch_modulus);
 			opt_save = OPTION_DEFAULT;
 			if (punch_modulus <= 0)
 			{
@@ -234,7 +233,7 @@ read_transport(void)
 			str_tolower(token);
 			if (i == DIGIT)
 			{
-				sscanf(token, "%d", &bcon_first);
+				(void)sscanf(token, "%d", &bcon_first);
 				if (bcon_first < 1 || bcon_first > 3)
 				{
 					input_error++;
@@ -264,7 +263,7 @@ read_transport(void)
 			str_tolower(token);
 			if (i == DIGIT)
 			{
-				sscanf(token, "%d", &bcon_last);
+				(void)sscanf(token, "%d", &bcon_last);
 				if (bcon_last < 1 || bcon_last > 3)
 				{
 					input_error++;
@@ -293,7 +292,7 @@ read_transport(void)
 		case 5:					/* timest */
 		case 14:				/* time_step */
 			if (copy_token(token, &next_char, &l) == DIGIT)
-				sscanf(token, SCANFORMAT, &timest);
+				(void)sscanf(token, SCANFORMAT, &timest);
 			{
 				std::string stdtoken;
 				j = copy_token(stdtoken, &next_char);
@@ -304,7 +303,7 @@ read_transport(void)
 				}
 				if (j == DIGIT)
 				{
-					sscanf(stdtoken.c_str(), SCANFORMAT, &mcd_substeps);
+					(void)sscanf(stdtoken.c_str(), SCANFORMAT, &mcd_substeps);
 				}
 			}
 			//if (copy_token(token, &next_char, &l) == DIGIT)
@@ -319,7 +318,7 @@ read_transport(void)
 			break;
 		case 6:				/* diffc */
 		case 16:				/* diffusion_coefficient */
-			sscanf(next_char, SCANFORMAT, &diffc);
+			(void)sscanf(next_char, SCANFORMAT, &diffc);
 			opt_save = OPTION_DEFAULT;
 			break;
 		case 7:				/* tempr */
@@ -327,7 +326,7 @@ read_transport(void)
 		case 19:				/* temperature_retardation_factor */
 		case 39:				/* thermal_diffusion */
 			if (copy_token(token, &next_char, &l) == DIGIT)
-				sscanf(token, SCANFORMAT, &tempr);
+				(void)sscanf(token, SCANFORMAT, &tempr);
 			if (tempr < 1)
 			{
 				tempr = 1;
@@ -337,7 +336,7 @@ read_transport(void)
 			}
 			j = copy_token(token, &next_char, &l);
 			if (j == DIGIT)
-				sscanf(token, SCANFORMAT, &heat_diffc);
+				(void)sscanf(token, SCANFORMAT, &heat_diffc);
 			opt_save = OPTION_DEFAULT;
 			break;
 		case 8:				/* length */
@@ -376,7 +375,7 @@ read_transport(void)
 			if (copy_token(token, &next_char, &l) != EMPTY)
 			{
 				/* exchange factor */
-				if (sscanf(token, "%d", &(stag_data->count_stag)) != 1)
+				if (sscanf(token, "%d", &(stag_data.count_stag)) != 1)
 				{
 					input_error++;
 					error_string = sformatf(
@@ -389,7 +388,7 @@ read_transport(void)
 				j = copy_token(token, &next_char, &l);
 				if (j != EMPTY)
 				{
-					if (sscanf(token, SCANFORMAT, &(stag_data->exch_f)) != 1)
+					if (sscanf(token, SCANFORMAT, &(stag_data.exch_f)) != 1)
 					{
 						input_error++;
 						error_string = sformatf(
@@ -398,7 +397,7 @@ read_transport(void)
 						break;
 					}
 					copy_token(token, &next_char, &l);
-					if (sscanf(token, SCANFORMAT, &(stag_data->th_m)) != 1)
+					if (sscanf(token, SCANFORMAT, &(stag_data.th_m)) != 1)
 					{
 						input_error++;
 						error_string = sformatf(
@@ -407,7 +406,7 @@ read_transport(void)
 						break;
 					}
 					copy_token(token, &next_char, &l);
-					if (sscanf(token, SCANFORMAT, &(stag_data->th_im)) != 1)
+					if (sscanf(token, SCANFORMAT, &(stag_data.th_im)) != 1)
 					{
 						input_error++;
 						error_string = sformatf(
@@ -442,21 +441,21 @@ read_transport(void)
 			opt_save = OPTION_DEFAULT;
 			break;
 		case 26:				/* dump */
+		{
 			dump_in = TRUE;
-			next_char_save = next_char;
-			if (copy_token(file_name, &next_char, &l) == EMPTY)
-				strcpy(file_name, "phreeqc.dmp");
-			else
+			std::string temp_name(next_char);
+			string_trim(temp_name);
+			if (temp_name.size() > 0)
 			{
-				string_trim(next_char_save);
-				strcpy(file_name, next_char_save);
+				file_name = temp_name;
 			}
 			opt_save = OPTION_DEFAULT;
 			break;
+		}
 		case 27:				/* output */
 		case 28:				/* output_frequency */
 		case 34:				/* print_frequency */
-			sscanf(next_char, "%d", &print_modulus);
+			(void)sscanf(next_char, "%d", &print_modulus);
 			opt_save = OPTION_DEFAULT;
 			if (print_modulus <= 0)
 			{
@@ -469,7 +468,7 @@ read_transport(void)
 		case 31:				/* dump_frequency */
 			dump_in = TRUE;
 			if (copy_token(token, &next_char, &l) == DIGIT)
-				sscanf(token, "%d", &dump_modulus);
+				(void)sscanf(token, "%d", &dump_modulus);
 			else
 			{
 				warning_msg("Expected integer value for dump_frequency.");
@@ -480,7 +479,7 @@ read_transport(void)
 		case 32:				/* dump_restart */
 			dump_in = TRUE;
 			if (copy_token(token, &next_char, &l) == DIGIT)
-				sscanf(token, "%d", &transport_start);
+				(void)sscanf(token, "%d", &transport_start);
 			else
 			{
 				warning_msg
@@ -495,7 +494,7 @@ read_transport(void)
 			break;
 		case 36:				/* initial_time */
 			if (copy_token(token, &next_char, &l) == DIGIT)
-				sscanf(token, SCANFORMAT, &initial_total_time);
+				(void)sscanf(token, SCANFORMAT, &initial_total_time);
 			{
 				std::string stdtoken;
 				j = copy_token(stdtoken, &next_char);
@@ -678,7 +677,7 @@ read_transport(void)
 		case 45:                /* current     */
 			if (copy_token(token, &next_char, &l) == DIGIT)
 			{
-				sscanf(token, SCANFORMAT, &fix_current);
+				(void)sscanf(token, SCANFORMAT, &fix_current);
 //				fix_current = fabs(fix_current);
 			}
 			else
@@ -704,7 +703,7 @@ read_transport(void)
 			}
 			if (copy_token(token, &next_char, &l) == DIGIT)
 			{
-				sscanf(token, SCANFORMAT, &max_mixf);
+				(void)sscanf(token, SCANFORMAT, &max_mixf);
 			}
 			else
 			{
@@ -744,8 +743,8 @@ read_transport(void)
 		max_cells = count_length;
 	if (count_disp > max_cells)
 		max_cells = count_disp;
-	if (count_por > max_cells * (1 + stag_data->count_stag))
-		max_cells = (int)ceil(((double)count_por / (double)(1 + stag_data->count_stag)));
+	if (count_por > max_cells * (1 + stag_data.count_stag))
+		max_cells = (int)ceil(((double)count_por / (1 + (double)stag_data.count_stag)));
 	if (max_cells > count_cells)
 	{
 		if (max_cells == count_length)
@@ -766,18 +765,17 @@ read_transport(void)
 		{
 			sprintf(token,
 				"Number of mobile cells is increased to (ceil)(number of porosities) / (1 + number of stagnant zones) = %d.",
-				(int) ceil(((double)count_por / (double)(1 + stag_data->count_stag))));
+				(int) ceil(((double)count_por / (1 + (double)stag_data.count_stag))));
 			warning_msg(token);
 		}
 	}
 	/*
 	*   Allocate space for cell_data
 	*/
-	int all_cells_now = max_cells * (1 + stag_data->count_stag) + 2;
-	space((void **)((void *)&cell_data), all_cells_now, &cell_data_max_cells,
-		sizeof(struct cell_data));
-
-	// initialize new cells
+	int all_cells_now = max_cells * (1 + stag_data.count_stag) + 2;
+	cell_data.resize(all_cells_now); // new classes initialized by global_structures.h
+	// But first two previously allocated for Change_Surf, so may
+	// need to reinitialize
 	if (all_cells_now > all_cells)
 	{
 		for (int i = all_cells; i < all_cells_now; i++)
@@ -793,8 +791,8 @@ read_transport(void)
 			cell_data[i].print = FALSE;
 			cell_data[i].same_model = FALSE;
 		}
-		all_cells = all_cells_now;
 	}
+	all_cells = all_cells_now;
 
 	/*
 	*   Fill in data for lengths
@@ -822,7 +820,7 @@ read_transport(void)
 				"Cell-lengths were read for %d cells. Last value is used till cell %d.",
 				count_length, max_cells);
 			warning_msg(error_string);
-			for (i = count_length; i <= max_cells; i++)
+			for (size_t i = count_length; i <= max_cells; i++)
 				cell_data[i + 1].length = length[count_length - 1];
 		}
 	}
@@ -888,15 +886,15 @@ read_transport(void)
 	}
 	else
 	{
-		if ((stag_data->exch_f > 0) && (stag_data->count_stag == 1))
+		if ((stag_data.exch_f > 0) && (stag_data.count_stag == 1))
 		{
 			error_string = sformatf(
 				"Mobile porosities were read, but mobile/immobile porosity was also defined in -stagnant. Using the values from -stagnant for mobile/immobile exchange and tortuosity factors.");
 			warning_msg(error_string);
 			for (i = 1; i <= max_cells; i++)
-				cell_data[i].por = stag_data->th_m;
+				cell_data[i].por = stag_data.th_m;
 			for (i++; i <= 2 * max_cells + 1; i++)
-				cell_data[i].por = stag_data->th_im;
+				cell_data[i].por = stag_data.th_im;
 		}
 		else
 		{
@@ -908,16 +906,16 @@ read_transport(void)
 			}
 			if (all_cells - 2 > count_por)
 			{
-				int st = stag_data->count_stag ? 1 : 0;
+				int st = stag_data.count_stag ? 1 : 0;
 				error_string = sformatf(
 					"Porosities were read for %d cells. Last value is used till cell %d.",
 					count_por, all_cells - st);
 				warning_msg(error_string);
 				for (i = count_por; i < all_cells - st; i++)
 				{
-					if (i == max_cells)
-						continue;
-					assert((i+1) < all_cells);
+					//if (i == max_cells)
+					//	continue;
+					//assert((i+1) < all_cells);
 					if ((i+1) < all_cells)
 					{
 						cell_data[i + 1].por = pors[count_por - 1];
@@ -943,12 +941,12 @@ read_transport(void)
 	/*
 	*  Account for stagnant cells
 	*/
-	if (stag_data->count_stag > 0)
+	if (stag_data.count_stag > 0)
 	{
-		max_cells = count_cells * (1 + stag_data->count_stag) + 2;
+		max_cells = count_cells * (1 + stag_data.count_stag) + 2;
 		for (i = 1; i <= count_cells; i++)
 		{
-			for (l = 1; l <= stag_data->count_stag; l++)
+			for (l = 1; l <= stag_data.count_stag; l++)
 				cell_data[i + 1 + l * count_cells].mid_cell_x =
 				cell_data[i].mid_cell_x;
 		}
@@ -1093,9 +1091,9 @@ read_transport(void)
 	*/
 	if (heat_diffc < 0)
 		heat_diffc = diffc;
-	else if (stag_data->count_stag == 1)
+	else if (stag_data.count_stag == 1)
 	{
-		if (stag_data->exch_f > 0)
+		if (stag_data.exch_f > 0)
 		{
 			if (diffc <= 0 && heat_diffc > 0)
 			{
@@ -1123,7 +1121,7 @@ read_transport(void)
 			}
 		}
 	}
-	else if (stag_data->count_stag > 1 && heat_diffc > diffc)
+	else if (stag_data.count_stag > 1 && heat_diffc > diffc)
 	{
 		input_error++;
 		error_string = sformatf(
@@ -1150,7 +1148,7 @@ read_transport(void)
 
 /* ---------------------------------------------------------------------- */
 int Phreeqc::
-read_line_LDBLEs(char *next_char, LDBLE ** d, int *count_d, int *count_alloc)
+read_line_LDBLEs(const char* next_char, LDBLE ** d, int *count_d, int *count_alloc)
 /* ---------------------------------------------------------------------- */
 {
 	int i, j, l, n;
@@ -1171,7 +1169,7 @@ read_line_LDBLEs(char *next_char, LDBLE ** d, int *count_d, int *count_alloc)
 		}
 		else
 		{
-			sscanf(token, SCANFORMAT, &value);
+			(void)sscanf(token, SCANFORMAT, &value);
 			n = 1;
 		}
 		for (;;)
