@@ -63,6 +63,15 @@
 #include "smalldense.h"
 #include "sundialstypes.h"
 #include "sundialsmath.h"
+
+#if defined(PHREEQCI_GUI)
+#ifdef _DEBUG
+#define new DEBUG_NEW
+#undef THIS_FILE
+static char THIS_FILE[] = __FILE__;
+#endif
+#endif
+
 /* WARNING don`t include any headers below here */
 
 #define ZERO RCONST(0.0)
@@ -83,7 +92,7 @@ denalloc(integertype n)
 	if (a == NULL)
 		return (NULL);
 
-	a[0] = (realtype *) malloc(n * n * sizeof(realtype));
+	a[0] = (realtype *) malloc((size_t)n * (size_t)n * sizeof(realtype));
 	if (a[0] == NULL)
 	{
 		free(a);
@@ -91,7 +100,7 @@ denalloc(integertype n)
 	}
 
 	for (j = 1; j < n; j++)
-		a[j] = a[0] + j * n;
+		a[j] = a[0] + (size_t)j * (size_t)n;
 
 	return (a);
 }
